--[[
Name: HealPoints
Description: Healer gear benchmark
Revision: $Revision: 114 $
Author: Eridan
Email: joh AT idi DOT ntnu DOT no
LastUpdate: $Date: 2012-02-01 23:57:24 +0000 (Wed, 01 Feb 2012) $

http://www.wowace.com/addons/heal-points/
Public SVN: svn://svn.wowace.com/wow/heal-points/mainline/trunk
]]

---------------
-- Libraries --
---------------
local addonName = "HealPoints";
local L = LibStub("AceLocale-3.0"):GetLocale(addonName) --Get the localization for our addon
local StatLogic = LibStub("LibStatLogic-1.2")
local Toolkit = LibStub("Toolkit-1.0", 2);

--------------------
-- AceAddon Setup --
--------------------
-- AceAddon Initialization

HealPoints = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0", "AceDebug-3.0", "AceTimer-3.0")
local HealPoints = HealPoints

HealPoints.version = GetAddOnMetadata(addonName, "Version");
HealPoints.version = HealPoints.version.." (r"..gsub("$Revision: 114 $", "$Revision: (%d+) %$", "%1")
--HealPoints.version = HealPoints.version.." ($Revision: 114 $)";
--Append "a" (alpha) to revision if it's alpha. The wowace packager will convert alpha..end-alpha into a block level comment
--@alpha@
HealPoints.version = HealPoints.version.."a"
--@end-alpha@
HealPoints.version = HealPoints.version..")"
HealPoints.date = gsub("$Date: 2012-02-01 23:57:24 +0000 (Wed, 01 Feb 2012) $", "^.-(%d%d%d%d%-%d%d%-%d%d).-$", "%1")


--[[
Properties
==========
HealPoints.ISHEALER
	Is set to 1 if the player is a healer (Paladin, Priest, Druid, Shaman)
	otherwise nil
	Initialized during OnInitialize method which is called by WowAce framework during the ADDON_LOADED event.
--]]
HealPoints.ISHEALER = nil; --Default ISHEALER to nil until we know for sure
HealPoints.playerStats = {}; --a table of the player's current stats
	--[[
		The following values are filled with a call to GetCurrentPlayerStats(stats):
			data.playerLevel  		e.g. 85
			data.playerClass  		e.g. "Paladin"
			data.strength
			data.agility
			data.stamina
			data.intellect
			data.spirit
			
			data.mana				Player's maximum mana
			data.baseMana			Player's base mana (amount with zero intellect or mana increasing effects; determines "base mana regen")
			data.normalManaRegen	Player's mana gained (per second) while not in combat
			data.baseManaRegen 		Player's "base" mana regeneration (i.e. "Combat Regen" without meditation)
			data.combatManaRegen	Player's combat mana regeneration (i.e. "Combat Regen" tooltip value)
			data.manaCostModifier	Defaults to 1.0.

			data.spellPower			Players tooltip "Spell Power" e.g. 3138
			data.spellCritRating 	Player's spell "Crit Rating" e.g. Crit rating 263 (+4.36% crit chance) 
			data.spellCritChance	Player's tooltip "Crit Chance" e.g. 16.04%
			data.spellHasteRating	Player's spell "Haste Rating" e.g. Haste rating 487 (+11.31% Haste)
			data.spellHaste			Player's spell "Haste" (e.g. Haste +11.31%)
			
			data.attackPower		Player's attack power, e.g. 399 (Increases damage with melee weapons by 28.5 damage per second.)
	--]]

---------------------
-- Initializations --
---------------------
--[[ Loading Process Event Reference
{
ADDON_LOADED - When this addon is loaded (exposed as :OnInitialize)
VARIABLES_LOADED - When all addons are loaded
PLAYER_LOGIN - Most information about the game world should now be available to the UI (exposed as :OnEnable)
}
--]]

-- Default values
local defaults = {};
	defaults.char = {};
		defaults.char.power = {};
		defaults.char.endurance = {};
		defaults.char.hot = {};
		defaults.char.gems = {};
		defaults.char.power.spell = "";
		defaults.char.power.auto = true;
		defaults.char.power.duration = 1;
		defaults.char.power.rank = nil; -- Old stat
		defaults.char.power.mana = 100;
		defaults.char.endurance.spell = "";
		defaults.char.endurance.auto = true;
		defaults.char.endurance.duration = 5;
		defaults.char.endurance.rank = nil; -- Old stat
		defaults.char.endurance.mana = 0;
		defaults.char.hot.numtargets = 3;
		defaults.char.tooltips = 1;
		defaults.char.ignoreGemsInTooltipDiff = true;
		defaults.char.ignoreEnchantsInTooltipDiff = true;
		defaults.char.ignorePrismaticInTooltipDiff = true;
		defaults.char.debug = false;
		defaults.char.debugLevel = 0;
		defaults.char.gems.blueGem = 1;
		defaults.char.gems.redGem = 1;
		defaults.char.gems.yellowGem = 1;
	
		defaults.char.replenishment = false;
		defaults.char.divineplea = false;


--- OnInitialize(name) called at ADDON_LOADED by WowAce 
function HealPoints:OnInitialize()
	self:LevelDebug(3, "HealPoints:OnInitialize()");
	self.db = LibStub("AceDB-3.0"):New("HEALPOINTS_CONFIG", defaults);

	self:SetDebugging(self.db.char.debug);

	--self:Print("OnInitialize");
	--self:Print(self.db.char.debugLevel or 0);
	local debugLevel = self.db.char.debugLevel;
	if (debugLevel == 0) then
		debugLevel = nil;
	end;
	self:SetDebugLevel(debugLevel);

	local _, className = UnitClass("player");
	if (className == "PALADIN" or className == "PRIEST" or className == "DRUID" or className == "SHAMAN") then
		HealPoints.ISHEALER = 1;
		HealPointsSpells:init();
		HealPointsCalculatorUI:init();
		HealPointsConfigUI:init();
	else
		return;
	end

	--Call SetupOptions if we've included the options file. (Not like there's any reason not to include it, its not like you can use the addon. But it's modular, helps with testing)
	if (self.SetupOptions) then
		self:SetupOptions() --in HealPoints_Options.lua
	end

	--Register LibDataBroker objects if we've included the file. Modularity!
	if (self.RegisterLDBDataObjects) then
		self:RegisterLDBDataObjects(); --in HealPointsFu.lua
	end;
end

--- OnEnable() called at PLAYER_LOGIN by WowAce
function HealPoints:OnEnable(first)
	self:LevelDebug(3, "HealPoints:OnEnable()")
	
	--If this character isn't a healer (e.g. Paladin, Priest, Shaman, Druid) then we can unload ourselves straight away
	if (HealPoints.ISHEALER ~= 1) then
		DisableAddOn("HealPoints");
		return;
	end;
	
	self:RegisterEvent("UNIT_AURA", "StatsChanged"); 
			--fires before the effect is in place. 
			--But we have to use it because things like Blessing of Might (which gives +mana/5) doesn't count as a stat change
			--and there is no event for regen changing
			--To get the event after the effect has happened track the real event 
			--e.g. UNIT_STATS, UNIT_ATTACK_POWER
	self:RegisterEvent("UNIT_LEVEL", "StatsChanged");
	--self:RegisterEvent("UNIT_MAXMANA", "StatsChanged"); event removed in 4.3
	self:RegisterEvent("UNIT_STATS", "StatsChanged");
	self:RegisterEvent("UNIT_SPELL_HASTE", "StatsChanged"); --not documented
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "SpellsChanged");
	--self:RegisterEvent("CHARACTER_POINTS_CHANGED", "TalentsChanged");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "TalentsChanged"); 
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "TalentsChanged"); 
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "TalentsChanged"); 

--	if (not self:IsHooked(HealPointsBS, "update")) then
--		self:SecureHook(HealPointsBS, "update", HealPoints.GearChanged);
--	end
	--HealPointsConfigUI:spellsChanged();
--	if (not first) then
		--HealPointsCalculator:updateHealPoints();
	--end
		
	-- Hook item tooltips
	if HealPointsTooltips then
		HealPointsTooltips:OnEnable();
	end;

	-- Add "HealPoints" section to the character paperdoll
	if (HealPointsCharUI) then
		HealPointsCharUI:AddStatFrames()
		--HealPoints_CharFrame:Show();
	end;

	if (HealPoints.RegisterLDBDataObjects) then
		HealPoints:RegisterLDBDataObjects();
	end;
end

function HealPoints:OnDisable()
	HealPoints_CharFrame:Hide();
	HealPoints_CalcFrame:Hide();
	HealPoints_ConfigFrame:Hide();
	
	-- Unhook item tooltips
	if HealPointsTooltips then
		HealPointsTooltips:Disable();
	end;
end

--- This method is called whenever one of the player's stats changes.
function HealPoints:StatsChanged(event, unitID) -- UNIT_AURA, UNIT_LEVEL, UNIT_MAXMANA, UNIT_STATS
	if (UnitIsUnit(unitID, "player")) then
		HealPoints:UpdateHealPoints((event or "").." "..(unitID or ""));
	end
end

function HealPoints:SpellsChanged(spellID, tabID) -- LEARNED_SPELL_IN_TAB
	HealPointsSpells:spellsChanged();
	HealPointsCalculatorUI:updateSpellLists();
	HealPointsConfigUI:spellsChanged();

	HealPoints:UpdateHealPoints("spellID "..(spellID or ""));
end

function HealPoints:TalentsChanged(delta) -- CHARACTER_POINTS_CHANGED
	--HealPointsCalculator:talentsChanged();  this function has been removed
	HealPointsCalculatorUI:updateSpellLists();

	HealPoints:UpdateHealPoints("TalentsChanged");
	HealPoints:LevelDebug(1, "TalentsChanged (%s)", delta or "");
end

function HealPoints:GearChanged()
	HealPoints:UpdateHealPoints("GearChanged");
end

---Something global has changed, and healpoints need to be updated, 
-- whether they are in the calculator, character sheet, or tooltip.
local tmrUpdate;
function HealPoints:UpdateHealPoints(sender)
	HealPoints:ClearCache(sender);

	--if (tmrUpdate) then
--		self:CancelTimer(tmrUpdate, true);
--	end;

	--self:Print("UpdateHealPoints() - setting timer");
	if (tmrUpdate == nil) then
		tmrUpdate = self:ScheduleTimer("EndUpdateHealPoints", 0.100, sender); --50ms = 20fps.  60fps = 16.666ms
	end;
end;

function HealPoints.EndUpdateHealPoints(event, sender)
	--HealPoints:Print("Timer Fire - EndUpdateHealPoints("..Toolkit:VarAsString(sender)..")");
	tmrUpdate = nil;

	if (HealPoints_CalcFrame:IsVisible()) then
		HealPointsCalculatorUI:UpdateCalculator(sender);
	end

	--if (CharacterFrame:IsVisible()) then
	--	HealPointsCalculator:updateHealPoints();
	--end

	--Only update LDB if we loaded it
	if (HealPoints.UpdateLdb) then
		HealPoints:UpdateLdb();
	end;
end;

function HealPoints:listScannedBonuses()
	local _, className = UnitClass("player");
	self:Print("Special set/equip bonuses detected:");
	self:Print("===================================");
	if (className == "PALADIN") then
		self:Print("Increases the effective spell power of your Holy Shock when used as a healing spell by "..HealPointsBS:GetBonus('AVG_PC_HS').."%");
		self:Print("Increases spell power of Flash of Light by "..HealPointsBS:GetBonus('AVG_PC_FOL').."%");
		self:Print("The cost of your Holy Light is reduced by "..HealPointsBS:GetBonus('MANA_PC_HL').."%");
		self:Print("Increases spell power of Flash of Light by "..HealPointsBS:GetBonus('AVG_ABS_FOL'));
		self:Print("Increases spell power of Holy Light by "..HealPointsBS:GetBonus('AVG_ABS_HL'));
		self:Print("Reduces the mana cost of Holy Light by "..HealPointsBS:GetBonus('MANA_ABS_HL'));
	elseif (className == "PRIEST") then
		self:Print("Increases the duration of your Renew spell by "..HealPointsBS:GetBonus('DURATION_RENEW').." sec.");
		self:Print("Reduces the mana cost of your Prayer of Healing ability by "..HealPointsBS:GetBonus('MANA_PC_POH').."%");
		self:Print("Increases the healing from your Greater Heal ability by "..HealPointsBS:GetBonus('AVG_PC_GH').."%");
		self:Print("The cost of your Greater Heal is reduced by "..HealPointsBS:GetBonus('MANA_PC_GH').."%");
	elseif (className == "DRUID") then
		self:Print("Increases the duration of your Regrowth spell by "..HealPointsBS:GetBonus('DURATION_REGR').." sec.");
		self:Print("Increases the spell power of the final healing value of your Lifebloom by "..HealPointsBS:GetBonus('AVG_BURST_LIFEBL'));
		self:Print("Increases the healing from your Healing Touch ability by "..HealPointsBS:GetBonus('AVG_PC_HT').."%");
		self:Print("The cost of your Rejuvenation is reduced by "..HealPointsBS:GetBonus('MANA_PC_REJUV').."%");
		self:Print("Increases the amount healed by Healing Touch by "..HealPointsBS:GetBonus('AVG_ABS_HT'));
		self:Print("Gain up to "..HealPointsBS:GetBonus('MANA_REFUND_HT').." mana each time you cast Healing Touch ");
		self:Print("Increases spell power of Rejuvenation by "..HealPointsBS:GetBonus('AVG_ABS_REJUV'));
		self:Print("Increases the spell power on the periodic portion of your Lifebloom by "..HealPointsBS:GetBonus('AVG_HOT_LIFEBL'));
		self:Print("Reduces the mana cost of Regrowth by "..HealPointsBS:GetBonus('MANA_ABS_REGR'));
		self:Print("Reduces the mana cost of Rejuvenation by "..HealPointsBS:GetBonus('MANA_ABS_REJUV'));
	elseif (className == "SHAMAN") then
		self:Print("Reduces the cost of your Lesser Healing Wave spell by "..HealPointsBS:GetBonus('MANA_PC_LHW').."%");
		self:Print("Your Chain Heal ability costs "..HealPointsBS:GetBonus('MANA_PC_CHAIN').."% less mana.");
		self:Print("Increases the amount healed by your Chain Heal ability by "..HealPointsBS:GetBonus('AVG_PC_CHAIN').."%");
		self:Print("Increases the healing done by your Chain Heal and Healing Wave by "..HealPointsBS:GetBonus('AVG_PC_CHAIN_HW').."%");
		self:Print("Increases the base amount healed by Chain Heal by "..HealPointsBS:GetBonus('AVG_ABS_CHAIN'));
		self:Print("Increases spell power of Lesser Healing Wave by "..HealPointsBS:GetBonus('AVG_ABS_LHW'));
		self:Print("Reduces the mana cost of Heaing Wave by "..HealPointsBS:GetBonus('MANA_ABS_HW'));
		self:Print("Increases spell power of Healing Wave by "..HealPointsBS:GetBonus('AVG_ABS_HW'));
		self:Print("Reduces the mana cost of Chain Heal by "..HealPointsBS:GetBonus('MANA_ABS_CHAIN'));
	end
end

--- Toggle the visibility of the HealPoints calculator
function HealPoints:ToggleCalculator()
	if (HealPoints_CalcFrame:IsVisible()) then
		HealPoints_CalcFrame:Hide();
	else
		HealPointsCalculatorUI:UpdateCalculator();
		HealPoints_CalcFrame:Show();
	end
end;

--[[
Returns true if the user is pressing the configured "Suppress all the things!" key
--]]
function HealPoints:IsSuppressKeyPressed()
	return IsControlKeyDown();
end;

function HealPoints:ClearCache(sender)
	--TODO: something
	--lastLink = nil; --clear the link cache
	HealPointsCalculator:ClearCache(sender);
end;