﻿HealPointsCalculatorUI = { };

local L = LibStub("AceLocale-3.0"):GetLocale("HealPoints") --Get the localization for our addon
local Toolkit = LibStub("Toolkit-1.0", 2);

local HP = HealPoints;

local _suppressOnChange = 0;
local TH_TotalTimeCurrent = nil;
local TH_TotalTimeNew = nil;
local SELECTED_SPELL = nil;
local SpellClassIDs = { };
local SPELLLIST = { };

local function setStat(statLabel, text, delta)
	if Toolkit:NearlyEqual(delta, 0) then
		statLabel:SetText(text);
	elseif (delta > 0) then
		statLabel:SetText(GREEN_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
	else
		statLabel:SetText(RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
	end
end

function HealPointsCalculatorUI:setHealFactor(spell, factor1, factor2)
	local factorString = _G["HealPointsSpellInfo"];
	if (factor2 == nil) then
		factorString:SetText(L["1 healing spell power increases hp/cast by"].." "..format("%1.2f", factor1)..".");
	elseif (spell.class['type'] == 'regrowth') then
		factorString:SetText(L["1 healing spell power adds"].." "..format("%1.2f", factor1).." to burst and "..format("%1.2f", factor2).." to HoT.");
	elseif (spell.class['type'] == 'lifebloom') then
		factorString:SetText(L["1 healing spell power adds"].." "..format("%1.2f", factor1).." to HoT and "..format("%1.2f", factor2).." to burst.");
	end
end

--[[
	The Heal Points Calculator has three section:
		- Results
				HealPointsResult1Label
				HealPointsResult2Label
				HealPointsResult3Label
		- Spells
				xxx
				xxx
				xxx
		- Variables
				HealPointsVariables1Label: Intellect
				HealPointsVariables2Label:SetText("Spirit:");
				HealPointsVariables3Label:SetText("Max mana:");
				HealPointsVariables4Label:SetText("Normal regen/5s:");
				HealPointsVariables5Label:SetText("Combat regen/5s:");
				HealPointsVariables6Label:SetText("Healing spell power:");
				HealPointsVariables7Label:SetText("Spell critical rating:");
				HealPointsVariables8Label:SetText("Spell haste rating:");
	This function sets the "before" values of the lables in the Variables (3rd) section 
	using values from a passed in PlayerStats table
--]]
function HealPointsCalculatorUI:SetCurrentValues(playerStats)
	assert(playerStats ~= nil, "HealPointsCalculatorUI:SetCurrentValues - playerStats is nil");

	----------------------------
	-- First Section: Results --
	----------------------------
	

	----------------------------
	-- Second Section: Spells --
	----------------------------
	--because a lot of logic is combined into the "show after" routine, we'll leave it to there
	

	--Third Section: Variables
	_G["HealPointsVariables1Current"]:SetText(format("%5.0f", playerStats.intellect));
	_G["HealPointsVariables2Current"]:SetText(format("%5.0f", playerStats.spirit));
	_G["HealPointsVariables3Current"]:SetText(format("%5.0f", playerStats.mana));
	--_G["HealPointsVariables4Current"]:SetText(format("%5.0f", playerStats.normalManaRegen*5));
	_G["HealPointsVariables4Current"]:SetText(format("%5.0f", playerStats.combatManaRegen*5));
	_G["HealPointsVariables5Current"]:SetText(format("%5.0f", playerStats.spellPower));
	_G["HealPointsVariables6Current"]:SetText(format("%5.0f", playerStats.spellCritRating));
	_G["HealPointsVariables7Current"]:SetText(format("%5.0f", playerStats.spellHasteRating));
end

function HealPointsCalculatorUI:SetAfterValues(beforeStats, afterStats, spell)
	assert(beforeStats ~= nil, "HealPointsCalculatorUI:SetAfterValues - beforeStats is nil");
	assert(afterStats ~= nil, "HealPointsCalculatorUI:SetAfterValues - afterStats is nil");
	assert(spell ~= nil, "HealPointsCalculatorUI:SetAfterValues - spell is nil");

--	HealPoints:Print(Toolkit:VarAsString(spell));

	local r = function(n) return HealPointsUtil:round(n); end;

	----------------------------
	-- First Section: Results --
	----------------------------
	local function setResultInfo(labelNumber, name, currentStat, newStat)
		local diff = newStat - currentStat;
		_G["HealPointsResult"..labelNumber.."Label"]:SetText(name);

		_G["HealPointsResult"..labelNumber.."Current"]:SetText(Toolkit:IntToStrLocale(currentStat));
		setStat(_G["HealPointsResult"..labelNumber.."Difference"], Toolkit:IntToStrLocale(diff), diff);
		setStat(_G["HealPointsResult"..labelNumber.."Result"], Toolkit:IntToStrLocale(newStat), diff);

		--_G["HealPointsResult"..labelNumber.."Current"]:SetText(currentStat);
		--setStat(_G["HealPointsResult"..labelNumber.."Difference"], diff, diff);
		--setStat(_G["HealPointsResult"..labelNumber.."Result"], newStat, diff);
	end

	local function spellToString(spell)
		if (spell == nil) then
			return "";
		else
			local spellName = " ("..HealPointsSpells:GetSpellLink(spell, false)..")";
			return spellName;
		end
	end

	setResultInfo(1, L["HealPoints"]..":", r(beforeStats.healPoints), r(afterStats.healPoints), "%5.0f");
	setResultInfo(2, "= "..L["PowerPoints"]..spellToString(HealPointsCalculator.HP_LastPowerfulCurrent)..":", r(beforeStats.powerPoints), r(afterStats.powerPoints), "%5.0f");
	setResultInfo(4, "+ "..L["EndurancePoints"]..spellToString(HealPointsCalculator.HP_LastEfficientCurrent)..":", r(beforeStats.endurancePoints), r(afterStats.endurancePoints), "%5.0f");



	----------------------------
	-- Second Section: Spells --
	----------------------------
	--[[
				HealPointsSpell[n]Current
				HealPointsSpell[n]Difference
				HealPointsSpell[n]Result

		HP/s
		HP/mana
		HP/cast
		HP/manabar

		Cast time
		HP/tick
		Spell crit(%)	
--]]
	HealPointsCalculatorUI:setSpellStat(1, "HP/s:", r(beforeStats.hpSec), r(afterStats.hpSec), "%d");
	HealPointsCalculatorUI:setSpellStat(2, "HP/mana:", r(beforeStats.hpMana), r(afterStats.hpMana), "%1.1f");
	HealPointsCalculatorUI:setSpellStat(3, "HP/cast:", r(beforeStats.hpCast), r(afterStats.hpCast), "%d");
	HealPointsCalculatorUI:setSpellStat(4, "HP/manabar:", r(beforeStats.hpManaBar), r(afterStats.hpManaBar), "%d");

	-- Spell specific stats
	--HealPoints:Print("Print current values. spell="..Toolkit:VarAsString(spell));

	local spellCanCrit = ((spell['class'].type or "") ~= "hot") --Heal over Time spells (HoTs) cannot crit
	local spellHasCastTime = ((spell['class'].time or 0) > 0); --don't show a cast time if the spell has none
	local spellHasTicks = ((spell['class'].tickcount or 0) > 0) --don't show HP/tick if it doesn't tick

	HealPoints:LevelDebug(3, "   spellCanCrit (type <> hot) = %s", Toolkit:VarAsString(spellCanCrit));
	HealPoints:LevelDebug(3, "   spellHasCastTime (time > 0) = %s", Toolkit:VarAsString(spellHasCastTime));
	HealPoints:LevelDebug(3, "   spellHasTicks (tickcount > 0) = %s", Toolkit:VarAsString(spellHasTicks));

	local labelNumber = 5;

	if (spellCanCrit) then
		local oldCrit = beforeStats.spellCritChance;
		local newCrit = afterStats.spellCritChance;
		HealPointsCalculatorUI:setSpellStat(labelNumber, L["Spell crit (%):"], r(oldCrit), r(newCrit), "%1.2f");
		labelNumber = labelNumber+1;
	end;
	
	if (spellHasCastTime) then
		local oldCastTime = HealPointsSpells:GetCastTime(spell, beforeStats.spellHaste);
		local newCastTime = HealPointsSpells:GetCastTime(spell, afterStats.spellHaste);
		HealPointsCalculatorUI:setSpellStat(labelNumber, L["Cast time (s):"], r(oldCastTime), r(newCastTime),  "%1.2f");
		labelNumber = labelNumber+1;
	end

	if (spellHasTicks) then
		local oldHPTick = HealPointsCalculator:GetHPTick(spell, beforeStats.spellPower);
		local newHPTick = HealPointsCalculator:GetHPTick(spell, afterStats.spellPower);
		HealPointsCalculatorUI:setSpellStat(labelNumber, L["HP/tick:"], r(oldHPTick), r(newHPTick), "%d");
		labelNumber = labelNumber+1;
	end;

	HealPointsCalculatorUI:setSpellStat(labelNumber, L["Spell haste (%):"], beforeStats.spellHaste, afterStats.spellHaste, "%1.2f");
	labelNumber = labelNumber+1;
	
	while (labelNumber <= 8) do
		HealPointsCalculatorUI:setSpellStat(labelNumber, nil);
		labelNumber = labelNumber+1;
	end

	------------------------------------------------------------
	-- Third Section: Variables : HealPointsVariable[n]Result --
	------------------------------------------------------------
	HealPointsCalculatorUI:setVariableStatInfoAfter(1, r(afterStats.intellect), r(beforeStats.intellect), "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfoAfter(2, r(afterStats.spirit), r(beforeStats.spirit), "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfoAfter(3, r(afterStats.mana), r(beforeStats.mana), "%5.0f");
	--HealPointsCalculatorUI:setVariableStatInfoAfter(4, r(afterStats.normalManaRegen*5), r(beforeStats.normalManaRegen*5), "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfoAfter(4, r(afterStats.combatManaRegen*5), r(beforeStats.combatManaRegen*5), "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfoAfter(5, r(afterStats.spellPower), r(beforeStats.spellPower), "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfoAfter(6, r(afterStats.spellCritRating), r(beforeStats.spellCritRating), "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfoAfter(7, r(afterStats.spellHasteRating), r(beforeStats.spellHasteRating), "%5.0f");
end;

--Set label number x in the middle section (HealPointsSpellXxxx)
function HealPointsCalculatorUI:setSpellStat(labelNumber, name, oldValue, newValue, formatString)
	if (name == nil) then
		_G["HealPointsSpell"..labelNumber.."Label"]:SetText("");
		_G["HealPointsSpell"..labelNumber.."Current"]:SetText("");
		_G["HealPointsSpell"..labelNumber.."Difference"]:SetText("");
		_G["HealPointsSpell"..labelNumber.."Result"]:SetText("");
	elseif (oldValue < 0 or newValue < 0) then
		_G["HealPointsSpell"..labelNumber.."Label"]:SetText(name);
		_G["HealPointsSpell"..labelNumber.."Difference"]:SetText("");
		if (oldValue < 0) then
			_G["HealPointsSpell"..labelNumber.."Current"]:SetText("Infinite");
		else
			setStat(_G["HealPointsSpell"..labelNumber.."Current"], format(formatString, oldValue), 0);
		end
		if (newValue < 0) then
			_G["HealPointsSpell"..labelNumber.."Result"]:SetText("Infinite");
		else
			setStat(_G["HealPointsSpell"..labelNumber.."Result"], format(formatString, newValue), -1);
		end
	else
		local diff = newValue - oldValue;
		_G["HealPointsSpell"..labelNumber.."Label"]:SetText(name);
		setStat(_G["HealPointsSpell"..labelNumber.."Current"], format(formatString, oldValue), 0);
		setStat(_G["HealPointsSpell"..labelNumber.."Difference"], format(formatString, diff), diff);
		setStat(_G["HealPointsSpell"..labelNumber.."Result"], format(formatString, newValue), diff);
	end
end

--Set "after values" in the 3rd ("Variables") section
function HealPointsCalculatorUI:setVariableStatInfoAfter(labelNumber, newValue, oldValue, numberFormat)
	--local originalStatText = _G["HealPointsVariables"..number.."Current"];
	--local inputBox = _G["HealPointsVariables"..number.."InputBox"];
	local label = _G["HealPointsVariables"..labelNumber.."Result"];

	--local delta = inputBox:GetNumber() + extraDelta;
	--local newStat = currentStat + delta;
	--originalStatText:SetText(format(numberFormat, currentStat));
	setStat(label, format(numberFormat, newValue), (newValue-oldValue));
end

--[[
	This method is called by the frame whenever any of the input boxes has changed values
--]]
function HealPointsCalculatorUI:inputBoxChanged()
	HealPointsCalculatorUI:UpdateCalculator();
end;

--Don't call this function directly. Call UpdateCalculator(), which wraps it in a pcall (protected call)
local function UpdateCalculator_Unsafe()
	--HealPoints:Print("HealPointsCalculatorUI:UpdateCalculator()");

	local selectedSpell = HealPointsSpells:getSpell(SELECTED_SPELL);
	if (selectedSpell == nil) then
		return;
	end;

	--Get the player's current stats
	local currentStats = HealPointsCalculator:GetCurrentHealPoints(selectedSpell);
	
	--Display the "before" numbers
	HealPointsCalculatorUI:SetCurrentValues(currentStats);
	
	--Calculate new set of stats based on the current stats and the changes
	local changes = HealPointsCalculatorUI:GetChanges(); --get the input boxes into a set of changes

	--HealPoints:Print("changes="..Toolkit:VarAsString(changes));

	local afterStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	
	--Calculate heal points, using the currently selected spell
	HealPointsCalculator:CalculateHealPoints(afterStats, selectedSpell);

	--Write the "after" and healpoints values to the calculator
	HealPointsCalculatorUI:SetAfterValues(currentStats, afterStats, selectedSpell);
end;

function HealPointsCalculatorUI:UpdateCalculator()
	if (_suppressOnChange > 0) then
		return;
	end;
	--HealPoints:Print("UpdateCalculator");

	--_suppressOnChange = _suppressOnChange+1;

	--pcall(UpdateCalculator_Unsafe);
	UpdateCalculator_Unsafe();

	--_suppressOnChange = _suppressOnChange-1;
	--HealPoints:Print("UpdateCalculator-end");
end

--[[
	Fetch the values currently in the input boxes and return them as a "changes" table.
--]]
function HealPointsCalculatorUI:GetChanges()
	local changes = {};

	changes.intellect =        _G["HealPointsVariables1InputBox"]:GetNumber();
	changes.spirit =           _G["HealPointsVariables2InputBox"]:GetNumber();
	changes.mana =             _G["HealPointsVariables3InputBox"]:GetNumber();
	changes.combatManaRegen =  _G["HealPointsVariables4InputBox"]:GetNumber();
	changes.spellPower =       _G["HealPointsVariables5InputBox"]:GetNumber();
	changes.spellCritRating =  _G["HealPointsVariables6InputBox"]:GetNumber();
	changes.spellHasteRating = _G["HealPointsVariables7InputBox"]:GetNumber();

	return changes;
end;

function HealPointsCalculatorUI:updateSpellLists() -- Update dropdown menus
	local function updateSpellList(objref)
		for i = 1, table.getn(SPELLLIST), 1 do
			local info ={
				text = SPELLLIST[i],
				func = HealPointsCalculatorUI.spellSelected
			};
			if (objref and type(objref) == "table") then
				info.owner = objref;
			end
			UIDropDownMenu_AddButton(info)
		end
	end

	for k in pairs(SPELLLIST) do
		SPELLLIST[k] = nil;
	end
	for i = 1, table.getn(SpellClassIDs), 1 do
		if (HealPointsSpells:getHighestSpellRank(SpellClassIDs[i]) > 0) then -- TODO
			local spell = HealPointsSpells:getSpellByClassID(SpellClassIDs[i]);
			local spellName = GetSpellInfo(spell.id);
			table.insert(SPELLLIST, spellName);
		end
	end

	UIDropDownMenu_Initialize(HealPointsSpellSelectSpell,function() updateSpellList(HealPointsSpellSelectSpell); end);
	UIDropDownMenu_SetSelectedID(HealPointsSpellSelectSpell, 1);
	
	SELECTED_SPELL = SPELLLIST[UIDropDownMenu_GetSelectedID(HealPointsSpellSelectSpell)];
	--HealPointsCalculatorUI:updateSelectedSpellStats();
	--HealPoints:Print("HealPointsCalculatorUI:spellSelected()");
	HealPointsCalculatorUI:UpdateCalculator();
end


function HealPointsCalculatorUI:spellSelected()
	if (self.owner) then
		UIDropDownMenu_SetSelectedID(self.owner, self:GetID());
		SELECTED_SPELL = SPELLLIST[UIDropDownMenu_GetSelectedID(HealPointsSpellSelectSpell)];
		
		--HealPointsCalculatorUI:updateSelectedSpellStats();
		--HealPoints:Print("HealPointsCalculatorUI:spellSelected()");
		HealPointsCalculatorUI:UpdateCalculator();
	end
end

--[[
function HealPointsCalculatorUI:setTimeToOOM(curTime, newTime)
	TH_TotalTimeCurrent = curTime;
	TH_TotalTimeNew = newTime;
end
--]]

function HealPointsCalculatorUI:setTooltip(self)
	local function spellToString(spell)
		if (spell == nil) then
			return L["No spell selected"];
		else
			local spellName = HealPointsSpells:GetSpellLink(spell, false);
			return spellName;
		end
	end

	local function totalTimeToString(curTime, newTime)
		if (curTime == null) then
			curTime = L["n/a"];
		elseif (curTime < 0) then
			curTime = L["Infinite"];
		else
			curTime = format("%d", curTime).."s";
		end
		if (newTime == null) then
			newTime = L["n/a"];
		elseif (newTime < 0) then
			newTime = L["Infinite"];
		else
			newTime = format("%d", newTime).."s";
		end
		return newTime.." ("..curTime..")";
	end

	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");

	if (self:GetID() == 1) then -- HealPoints
		GameTooltip:SetText(L["HealPoints"], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		--GameTooltip:AddLine("\nSelected spells:", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);

		local szLine = L["PowerPoints"].." "..PLAYERSTAT_SPELL_COMBAT..": "..spellToString(HealPointsCalculator.HP_LastPowerfulCurrent);
		if (HealPoints.db.char.power['auto'] == true) then
			szLine = szLine.." ("..L["auto"]..")";
		end;
		GameTooltip:AddLine(szLine);

		--endurance		
		local szLine = L["EndurancePoints"].." "..PLAYERSTAT_SPELL_COMBAT..": "..spellToString(HealPointsCalculator.HP_LastEfficientCurrent);
		if (HealPoints.db.char.endurance['auto'] == true) then
			szLine = szLine.." ("..L["auto"]..")";
		end;
		GameTooltip:AddLine(szLine);

	elseif (self:GetID() == 2) then -- PowerPoints
		GameTooltip:SetText("PowerPoints", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		if (HealPoints.db.char.power['auto'] == true) then
			GameTooltip:AddLine("Hitpoints healed in "..HealPoints.db.char.power['duration'].." minute(s) using your most");
			GameTooltip:AddLine("powerful spell, starting with "..HealPoints.db.char.power['mana'].."% mana.");
			GameTooltip:AddLine("Regeneration and 5 sec rule are taken into account.");
			GameTooltip:AddLine("\nPowerPoints spell:", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			GameTooltip:AddLine(spellToString(HealPointsCalculator.HP_LastPowerfulCurrent));
		else
			GameTooltip:AddLine("Hitpoints healed in "..HealPoints.db.char.power['duration'].." minute(s) using");
			GameTooltip:AddLine(HealPoints.db.char.power['spell']..", starting with "..HealPoints.db.char.power['mana'].."% mana.");
			GameTooltip:AddLine("Regeneration and 5 sec rule are taken into account.");
		end

	elseif (self:GetID() == 4) then -- EndurancePoints
		GameTooltip:SetText("EndurancePoints", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		if (HealPoints.db.char.endurance['auto'] == true) then
			GameTooltip:AddLine("Hitpoints healed in "..HealPoints.db.char.endurance['duration'].." minute(s) using your most");
			GameTooltip:AddLine("efficient spell, starting with "..HealPoints.db.char.endurance['mana'].."% mana.");
			GameTooltip:AddLine("Regeneration and 5 sec rule are taken into account.");
			GameTooltip:AddLine("\nEndurancePoints spell:", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			GameTooltip:AddLine(spellToString(HealPointsCalculator.HP_LastEfficientCurrent));
		else
			GameTooltip:AddLine("Hitpoints healed in "..HealPoints.db.char.endurance['duration'].." minute(s) using");
			GameTooltip:AddLine(HealPoints.db.char.endurance['spell']..", starting with "..HealPoints.db.char.endurance['mana'].."% mana.");
			GameTooltip:AddLine("Regeneration and 5 sec rule are taken into account.");
		end
	
	elseif (self:GetID() == 8) then -- Tot. healed with regen
		GameTooltip:SetText("HP/manabar", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Hitpoints healed with selected spell");
		GameTooltip:AddLine("using the whole mana bar.");
		GameTooltip:AddLine("Casting regeneration is taken into account.");
		GameTooltip:AddLine("\nTime taken until mana bar is spent:", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		--todo: GameTooltip:AddLine(totalTimeToString(TH_TotalTimeCurrent, TH_TotalTimeNew));
	elseif (self:GetID() == 13) then -- Intellect
		GameTooltip:SetText("Intellect", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Increases Mana");
		GameTooltip:AddLine("Increases Spell Power");
		GameTooltip:AddLine("Increases Spell Critical Hit");
	elseif (self:GetID() == 14) then -- Spirit
		GameTooltip:SetText("Spirit", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Increases Mana Regeneration");
	elseif (self:GetID() == 15) then -- Max Mana
		GameTooltip:SetText("Mana", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine(STAT_MANA_TOOLTIP);
	elseif (self:GetID() == 16) then -- Casting regen
		GameTooltip:SetText("Combat mana regeneration", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Mana regenerated every 5 seconds");
		GameTooltip:AddLine("while in combat.");
	elseif (self:GetID() == 17) then -- Normal regen
		GameTooltip:SetText("Normal mana regeneration", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Mana regenerated every 5 seconds");
		GameTooltip:AddLine("while not in combat.");
	elseif (self:GetID() == 18) then -- % regen while in combat
		GameTooltip:SetText("Combat Regeneration (%)", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Percentage of normal mana regeneration");
		GameTooltip:AddLine("that continues while in combat");
	elseif (self:GetID() == 19) then -- Healing Spell Power
		GameTooltip:SetText(STAT_SPELLPOWER, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine(STAT_SPELLHEALINGf_TOOLTIP);
	elseif (self:GetID() == 20) then -- Spell Crit rating
		GameTooltip:SetText("Spell critical strike rating", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Chance of spells doing extra damage.");
		GameTooltip:AddLine("Only includes bonuses from gear.");
	elseif (self:GetID() == 21) then -- Spell Haste rating todo: use GlobalStrings.STAT_HASTE_SPELL_TOOLTIP
		GameTooltip:SetText("Spell Haste Rating", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Increases spell casting speed.");
	else
		GameTooltip:SetText("frame id: "..self:GetID());
		GameTooltip:AddLine("name:", self.Name);
	end
	GameTooltip:Show();
end

function HealPointsCalculatorUI_OnLoad(self)
	tinsert(UISpecialFrames, "HealPoints_CalcFrame");
	UIPanelWindows["HealPoints_CalcFrame"] = nil;

	HealPointsResultFrameTitle:SetText("Results");
	HealPointsSpellFrameTitle:SetText("Spell details");
	HealPointsVariableFrameTitle:SetText("Player stats");

	_G["HealPointsVariables1Label"]:SetText("Intellect:");
	_G["HealPointsVariables2Label"]:SetText("Spirit:");
	_G["HealPointsVariables3Label"]:SetText("Max mana:");
	--_G["HealPointsVariables4Label"]:SetText("Mana Regen:");
	_G["HealPointsVariables4Label"]:SetText("Combat Regen:");
	_G["HealPointsVariables5Label"]:SetText("Healing spell power:");
	_G["HealPointsVariables6Label"]:SetText("Spell critical rating:");
	_G["HealPointsVariables7Label"]:SetText("Spell haste rating:");
end;


function HealPointsCalculatorUI:Show()
	--HealPoints:Print("HealPointsCalculatorUI:Show()");

	if (SELECTED_SPELL == nil) then
		self:updateSpellLists();
	end

	HealPointsCalculatorUI:UpdateCalculator();

	--HealPointsCalculatorUI:updateStats();
	--self:updateSelectedSpellStats();
	--HealPointsCalculator:updateHealPoints();
end

-- Global functions
--[[
function HealPointsCalculatorUI:UpdateCurrentStats() -- Update stats (int, spi, regen, ...)

	local currentStats; -- = HealPoints.playerStats;
	currentStats = HealPointsCalculator:GetCurre7ntPlayerStats();
	
	assert(currentStats ~= nil, "HealPointsCalculator:UpdateCurrentStats: HealPoints.playerStats is nil");

	HealPointsCalculator:CalculateHealPoints(currentStats);

	HealPointsCalculatorUI:SetCurrentValues(currentStats);

	--[-[
	HealPointsCalculatorUI:setVariableStatInfo(1, currentIntellect, extraInt, "%5.0f");
	intDelta = intDelta + extraInt;

	-- Spirit
	local _, currentSpirit, _, _ = UnitStat("player", 5);
	local extraSpi = BonusPercentSpi * _G["HealPointsVariables2InputBox"]:GetNumber();
	HealPointsCalculatorUI:setVariableStatInfo(2, currentSpirit, extraSpi, "%5.0f");

	-- Max mana
	local currentMana = UnitManaMax("player");
	local extraMana = _G["HealPointsVariables3InputBox"]:GetNumber() + 15 * intDelta;
	HealPointsCalculatorUI:setVariableStatInfo(3, currentMana, 15 * intDelta + extraMana, "%5.0f");

	-- Mana regen (casting, normal, %casting)
	local manaRegenNormal, manaRegenCasting, manaRegenNormalNew, manaRegenCastingNew, percentRegen = getCurrentAndNewManaRegen(0, 0, 0, 0);
	local manaRegenCastingDelta = manaRegenCastingNew - manaRegenCasting - _G["HealPointsVariables4InputBox"]:GetNumber();
	local manaRegenNormalDelta = manaRegenNormalNew - manaRegenNormal - _G["HealPointsVariables5InputBox"]:GetNumber();
	HealPointsCalculatorUI:setVariableStatInfo(4, manaRegenCasting, manaRegenCastingDelta, "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfo(5, manaRegenNormal, manaRegenNormalDelta, "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfo(6, 100 * percentRegen, 0, "%5.0f");

	-- Healing spell power
	local currentHealing, newHealing = getCurrentAndNewHealing(0, 0, 0, 0);
	HealPointsCalculatorUI:setVariableStatInfo(7, currentHealing, newHealing - currentHealing - _G["HealPointsVariables7InputBox"]:GetNumber(), "%5.0f");

	-- Spell crit rating
	HealPointsCalculatorUI:setVariableStatInfo(8, GetCombatRating(11), 0, "%5.0f");

	-- Spell haste rating
	HealPointsCalculatorUI:setVariableStatInfo(9, GetCombatRating(20), 0, "%5.0f");
end
--]]


function HealPointsCalculatorUI:updateSelectedSpellStats_Depricated()
--	local spell = HealPointsSpells:getSpell(SELECTED_SPELL);

	--HealPoints:Print("HealPointsCaluclatorUI:updateSelectedSpellStats");
	HealPointsCalculatorUI:UpdateCalculator();
	
	--HealPointsCalculator:displaySpellStats(spell);
end

function HealPointsCalculatorUI:init()
	HP:Debug("HealPointsCalculatorUI:init()");
	SpellClassIDs = HealPointsSpells.GetSpellClassIDs();
end