﻿	HealPointsConfigUI = { };
local L = LibStub("AceLocale-3.0"):GetLocale("HealPoints") --Get the localization for our addon
local HP = HealPoints;

local valueUpdating = 0;
local SPELLLIST = { };
local SpellClassIDs = { };

local function updateSpellLists()
	local function updateSpellList(objref, funct)
		for i = 1, table.getn(SPELLLIST), 1 do
			local info ={
				text = SPELLLIST[i],
				func = funct
			};
			if (objref and type(objref) == "table") then
				info.owner = objref;
			end
			UIDropDownMenu_AddButton(info)
		end
	end

	--empty out SPELLLIST table
	for k in pairs(SPELLLIST) do
		SPELLLIST[k] = nil;
	end


	for i = 1, table.getn(SpellClassIDs), 1 do
		if (HealPointsSpells:getHighestSpellRank(SpellClassIDs[i]) > 0) then 
			--HealPoints:Print(1, "SpellClassID: %d", SpellClassIDs[i]);
			local spell = HealPointsSpells:getSpellByClassID(SpellClassIDs[i]);
			assert(spell, string.format("Spell ClassID %d not found", SpellClassIDs[i]));
			local spellName = GetSpellInfo(spell.id);
			table.insert(SPELLLIST, spellName);
		end
	end

	if (HP.db.char.power['spell'] == '') then
		HP.db.char.power['spell'] = SPELLLIST[1];
	end
	if (HP.db.char.endurance['spell'] == '') then
		HP.db.char.endurance['spell'] = SPELLLIST[1];
	end

	UIDropDownMenu_Initialize(HealPointsPowerConfigManualSelectSpell,function()
		updateSpellList(HealPointsPowerConfigManualSelectSpell, HealPointsConfigUI.powerSpellSelected);
	end);
	UIDropDownMenu_Initialize(HealPointsEnduranceConfigManualSelectSpell,function()
		updateSpellList(HealPointsEnduranceConfigManualSelectSpell, HealPointsConfigUI.enduranceSpellSelected);
	end);
end

function HealPointsConfigUI:init()
	HP:Debug("HealPointsConfigUI:init()");
	SpellClassIDs = HealPointsSpells.GetSpellClassIDs();
	self:spellsChanged();
end

local function updateGUI() -- Updates GUI to match HEALPOINTS_CONFIG
	HP:Debug("updateGUI()");
	valueUpdating = 1;

	-- PowerPoints
	_G["HealPointsPowerConfigDuration"]:SetValue(HP.db.char.power['duration']);
	_G["HealPointsPowerConfigDurationText"]:SetText(HP.db.char.power['duration'].." mins");
	if (HP.db.char.power['auto'] == true) then
		_G["HealPointsPowerConfigAutoSelect"]:SetChecked(1);
		_G["HealPointsPowerConfigManualSelect"]:SetChecked(0);
	else
		_G["HealPointsPowerConfigAutoSelect"]:SetChecked(0);
		_G["HealPointsPowerConfigManualSelect"]:SetChecked(1);
	end
	--Power points spell combo box		
	if (UIDropDownMenu_GetSelectedID(HealPointsPowerConfigManualSelectSpell) ~= HealPointsUtil:getTableIndex(SPELLLIST, HealPoints.db.char.power['spell'])) then
		UIDropDownMenu_SetSelectedID(HealPointsPowerConfigManualSelectSpell, HealPointsUtil:getTableIndex(SPELLLIST, HealPoints.db.char.power['spell']));
	end
	_G["HealPointsPowerConfigMana"]:SetValue(HP.db.char.power['mana']);
	_G["HealPointsPowerConfigManaText"]:SetText(HP.db.char.power['mana'].."%");


	-- EndurancePoints
	_G["HealPointsEnduranceConfigDuration"]:SetValue(HP.db.char.endurance['duration']);
	_G["HealPointsEnduranceConfigDurationText"]:SetText(HP.db.char.endurance['duration'].." mins");
	if (HealPoints.db.char.endurance['auto'] == true) then
		_G["HealPointsEnduranceConfigAutoSelect"]:SetChecked(1);
		_G["HealPointsEnduranceConfigManualSelect"]:SetChecked(0);
	else
		_G["HealPointsEnduranceConfigAutoSelect"]:SetChecked(0);
		_G["HealPointsEnduranceConfigManualSelect"]:SetChecked(1);
	end
	--Endurance points spell combo box
	if (UIDropDownMenu_GetSelectedID(HealPointsEnduranceConfigManualSelectSpell) ~= HealPointsUtil:getTableIndex(SPELLLIST, HealPoints.db.char.endurance['spell'])) then
		UIDropDownMenu_SetSelectedID(HealPointsEnduranceConfigManualSelectSpell, HealPointsUtil:getTableIndex(SPELLLIST, HealPoints.db.char.endurance['spell']));
	end;
	_G["HealPointsEnduranceConfigMana"]:SetValue(HP.db.char.endurance['mana']);
	_G["HealPointsEnduranceConfigManaText"]:SetText(HP.db.char.endurance['mana'].."%");


	-- Heal-over-time
	local _, className = UnitClass("player");
	if (className == "PRIEST" or className == "DRUID") then
		_G["HealPointsHotConfigNumTargets"]:SetValue(HP.db.char.hot['numtargets']);
		_G["HealPointsHotConfigNumTargetsText"]:SetText(HP.db.char.hot['numtargets']);
	end

    if (HP.db.char.replenishment == true) then
        _G["HealPointsOptionsConfigReplenish"]:SetChecked(1);
    else
        _G["HealPointsOptionsConfigReplenish"]:SetChecked(0);
    end
    
    if (HP.db.char.divineplea == true) then
        _G["HealPointsOptionsConfigDivinePlea"]:SetChecked(1);
    else
        _G["HealPointsOptionsConfigDivinePlea"]:SetChecked(0);
    end    
    
	valueUpdating = 0;
end

function HealPointsConfigUI:powerSpellSelected()
	HP:Debug("powerSpellSelected()");
	if (self.owner) then
		UIDropDownMenu_SetSelectedID(self.owner, self:GetID());
		HP.db.char.power['spell'] = SPELLLIST[UIDropDownMenu_GetSelectedID(HealPointsPowerConfigManualSelectSpell)];
		--HealPointsCalculator:updateHealPoints();
		HP:UpdateHealPoints();
	end
end

function HealPointsConfigUI:enduranceSpellSelected()
	HP:Debug("enduranceSpellSelected()");
	if (self.owner) then
		UIDropDownMenu_SetSelectedID(self.owner, self:GetID());
		HP.db.char.endurance['spell'] = SPELLLIST[UIDropDownMenu_GetSelectedID(HealPointsEnduranceConfigManualSelectSpell)];
		--HealPointsCalculator:updateHealPoints();
		HP:UpdateHealPoints();
	end
end

function HealPointsConfigUI:update()
	HP:Debug("HealPointsConfigUI:update()");
	if (valueUpdating == 0) then
		-- PowerPoints
		HP.db.char.power['duration'] = _G["HealPointsPowerConfigDuration"]:GetValue();
		_G["HealPointsPowerConfigDurationText"]:SetText(HealPoints.db.char.power['duration'].." mins");
		if (_G["HealPointsPowerConfigAutoSelect"]:GetChecked() == 1) then
			HP.db.char.power['auto'] = true;
		else
			HP.db.char.power['auto'] = false;
		end
		HP.db.char.power['mana'] = _G["HealPointsPowerConfigMana"]:GetValue();
		_G["HealPointsPowerConfigManaText"]:SetText(HealPoints.db.char.power['mana'].."%");

		-- EndurancePoints
		HP.db.char.endurance['duration'] = _G["HealPointsEnduranceConfigDuration"]:GetValue();
		_G["HealPointsEnduranceConfigDurationText"]:SetText(HP.db.char.endurance['duration'].." mins");
		if (_G["HealPointsEnduranceConfigAutoSelect"]:GetChecked()) then
			HP.db.char.endurance['auto'] = true;
		else
			HP.db.char.endurance['auto'] = false;
		end
		HP.db.char.endurance['mana'] = _G["HealPointsEnduranceConfigMana"]:GetValue();
		_G["HealPointsEnduranceConfigManaText"]:SetText(HP.db.char.endurance['mana'].."%");

		-- Heal-over-time/AoE
		HP.db.char.hot['numtargets'] = _G["HealPointsHotConfigNumTargets"]:GetValue();
		local _, className = UnitClass("player");
		if (className == "PRIEST" or className == "DRUID") then
			_G["HealPointsHotConfigNumTargetsText"]:SetText(HP.db.char.hot['numtargets']);
		end
        
		--HealPointsCalculatorUI:updateSelectedSpellStats();
		--HealPointsCalculator:updateHealPoints();
		HP:UpdateHealPoints();
	end
end

function HealPointsConfigUI:reset()
	--HealPoints:ResetDB("char"); -- Doesn't work anymore for some reason
	HP.db.char.power['auto'] = true;
	HP.db.char.power['duration'] = 1;
	HP.db.char.power['rank'] = nil;
	HP.db.char.power['mana'] = 100;
	HP.db.char.endurance['auto'] = true;
	HP.db.char.endurance['duration'] = 5;
	HP.db.char.endurance['rank'] = nil;
	HP.db.char.endurance['mana'] = 0;
	HP.db.char.hot['numtargets'] = 3;
	HP.db.char.power['spell'] = SPELLLIST[1];
	HP.db.char.endurance['spell'] = SPELLLIST[1];
	updateGUI();

	--HealPointsCalculatorUI:updateSelectedSpellStats();
	--HealPointsCalculator:updateHealPoints();
	HP:UpdateHealPoints();
end

function HealPointsConfigUI:spellsChanged()
	HP:Debug("HealPointsConfigUI:spellsChanged()");
	updateSpellLists()
	updateGUI();
end

function HealPointsConfigUI:lifeblChanged()
	-- Replenishment
	if (_G["HealPointsOptionsConfigReplenish"]:GetChecked()) then
		HP.db.char.replenishment = true;
	else
		HP.db.char.replenishment = false;        
	end

	-- Divine Plea
	if (_G["HealPointsOptionsConfigDivinePlea"]:GetChecked()) then
		HP.db.char.divineplea = true;
	else
		HP.db.char.divineplea = false;        
	end

	--HealPointsCalculator:updateStats();
	--HealPointsCalculatorUI:updateSelectedSpellStats();
	--HealPointsCalculator:updateHealPoints();
	HP:UpdateHealPoints();
end

function HealPointsConfigUI:setTooltip(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if (self:GetID() == 1) then -- HoT config
		GameTooltip:SetText("Number of targets", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Number of targets you try to keep the HoT constantly running on (mana permitting) or number of targets for AoE heals.");
	elseif (self:GetID() == 2) then -- Start mana
		GameTooltip:SetText("Starting mana", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Percentage of mana available at the start of the computation.");
	elseif (self:GetID() == 3) then -- Replenishment
		GameTooltip:SetText("Replenishment", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Assume that the Replenishment buff is always active.");  
	elseif (self:GetID() == 4) then -- Divine Plea
		GameTooltip:SetText("Divine Plea", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Assume that Divine Plea is used on every cooldown.");  
	end
	GameTooltip:Show();
end


